//
//  TextDocument.m
//  Demo4
//
//  Created by 荻原 剛志 on 11/05/15.
//  Copyright 2011. All rights reserved.
//

#import "TextDocument.h"

static NSFont *defaultFont = nil;

@implementation TextDocument

- (NSString *)windowNibName { // Override returning the nib file name of the document
    return @"TextDocument";
}

- (void)windowControllerDidLoadNib:(NSWindowController *)aController
{
    [super windowControllerDidLoadNib:aController];
    // Add any code here that needs to be executed once the windowController has loaded the document's window.
    NSTextView *textView = [scrollView documentView];
    NSLayoutManager *layoutManager = [[NSLayoutManager alloc] init];
    NSString *txt = readData;
    if (txt == nil)
        txt = @"";
    else
        readData = nil;
    storage = [[NSTextStorage alloc] initWithString:txt];
    [storage addLayoutManager:layoutManager];
    [layoutManager addTextContainer:[textView textContainer]];
    [charsText setIntegerValue:[storage length]];
    [textView setEditable:YES];
    [textView setSelectable:YES];
    [textView setRichText:NO];
    if (defaultFont != nil)
        [textView setFont:defaultFont];
    [textView setDelegate:self];
}

- (NSData *)dataOfType:(NSString *)typeName error:(NSError **)outError {
    const char *utf = [[storage string] UTF8String];
    NSUInteger leng = strlen(utf);
    NSData *data = [[NSData alloc] initWithBytes:(void *)utf length:leng];
    if (data != nil)
        return data;
    if (outError) {
        *outError = [NSError errorWithDomain:NSOSStatusErrorDomain
                                        code:unimpErr userInfo:NULL];
    }
    return nil;
}

- (BOOL)readFromURL:(NSURL *)absoluteURL
             ofType:(NSString *)typeName error:(NSError **)outError
{
    NSStringEncoding enc;
    readData = [NSString stringWithContentsOfURL:absoluteURL
                                    usedEncoding:&enc error:outError];
    return (readData != nil);
}

- (NSPrintOperation *)printOperationWithSettings:(NSDictionary *)printSettings
                                           error:(NSError **)outError
{
    return [NSPrintOperation printOperationWithView:[scrollView documentView]];
}

- (BOOL)revertToContentsOfURL:(NSURL *)absoluteURL
                       ofType:(NSString *)typeName error:(NSError **)outError
{
    if ([self readFromURL:absoluteURL ofType:typeName error:outError]) {
        [storage setAttributedString:
            [[NSAttributedString alloc] initWithString:readData]];
        [self updateChangeCount:NSChangeCleared];
        return YES;
    }
    return NO;
}

- (void)textDidChange:(NSNotification *)aNotification {
    [charsText setIntegerValue:[storage length]];
}

- (NSDictionary *)textView:(NSTextView *)textView
    shouldChangeTypingAttributes:(NSDictionary *)oldTypingAttributes
                    toAttributes:(NSDictionary *)newTypingAttributes {
    NSFont *font = [newTypingAttributes objectForKey:NSFontAttributeName];
    if (font != nil)
        defaultFont = font;
    return newTypingAttributes;
}

@end
